/*
    StackOps.h
    Headerfile for an array-based implementation
    of a stack of primitive elements (int)
*/
#ifndef STACKOPS_H_INCLUDED
#define STACKOPS_H_INCLUDED
#define STACK_MAX_SIZE 65536
#define STACK_DEFAULT_SIZE 128

/**
* enumeration of stack operation results
*/
typedef enum {
    STACK_EMPTY = -1, /**< empty stack */
    NO_STACK, /**< no stack provided */
    STACK_OK /**< result ok */
} StackStateT;
typedef int StackElementT; // type of stack elements

/**
* Header cell for array-based dynamically growing stack
*/
typedef struct {
    StackElementT *memory; /**< memory to hold stack elements */
    size_t count; /**< number of elements currently on the stack */
    size_t size; /**< maximum number of elements the stack can currently accomodate */
    int top; /**< index of top of the stack; initial value should be STACK_EMPTY=-1 */
} StackT;

/**
* Stack operations
*/
extern StackT *createStack(size_t size);
extern StackT *deleteStack(StackT *s);
extern StackElementT pop(StackT *s, int *result);
extern int push(StackT *, StackElementT val);
extern StackElementT top(StackT *s, int *result);
/**
* Stack tests
*/
extern void testCreateStack();
extern void testPop();
extern void testPush();
extern void testTop();

#endif // STACKOPS_H_INCLUDED
